
import React, { useEffect, useState } from 'react';
import { IconButton, InputAdornment, makeStyles, TextField } from '@material-ui/core';
import { theme as StoryTheme, theme } from './Theme/theme';

/*Icons*/
import KeyboardArrowUpIcon from '@mui/icons-material/KeyboardArrowUp';
import KeyboardArrowDownIcon from '@mui/icons-material/KeyboardArrowDown';
import SearchIcon from '@mui/icons-material/Search';
import { Error } from '.';

type Value = string | number;
interface DataValue {
  text?: string
  value?: Value
  key?: string | number
};

export const Dropdown: React.FC<{
  id?: string,
  hideSearch?: boolean,
  value?: Value
  values?: DataValue[]
  type?: string
  prerequisite?: boolean
  emptyText?: string
  max?: number
  maxText?: string
  prerequisiteText?: string
  multiple?: boolean
  width?: number
  label?: string
  name?: string
  disabled?: boolean
  onChange?: (data: DataValue) => void
  onChangeValues?: (data: DataValue[]) => void
  onChangeInput?: React.ChangeEventHandler<HTMLInputElement> | undefined
  data?: DataValue[]
}> = ({
  id,
  hideSearch = false,
  value,
  values,
  type,
  prerequisite,
  emptyText,
  max,
  maxText,
  prerequisiteText,
  multiple,
  width,
  label,
  name,
  data,
  disabled,
  onChange,
  onChangeValues,
  onChangeInput,
  ...props
}) => {
    const [valueData, setValueData] = useState('');
    const [valuesData, setValuesData] = useState<DataValue[]>(values || []);
    const [upArrow, setUpArrow] = useState(false);
    const [search, setSearch] = useState('');
    const classes: any = styles({
      width,
      upArrow,
      border: upArrow ? 'none' : `1px solid ${StoryTheme.borderGreyPrimary}`,
      borderBottom: upArrow ? `1px solid ${StoryTheme.bluePrimary}` : `1px solid ${StoryTheme.borderGreyPrimary}`
    });
    const handleInputFilter = (e: React.ChangeEvent<HTMLInputElement>) => {
      if (onChangeInput) onChangeInput(e);
      setSearch(e.target.value);
    };
    const handleChange = (element: DataValue) => {
      if (multiple) {
        const _values: DataValue[] = [...valuesData];
        const index = _values.findIndex(v => v.value === element.value);
        if (index !== -1) return;
        _values.push(element);
        onChangeValues && onChangeValues(_values);
        setValuesData(_values);
      } else {
        onChange && onChange(element);
        setValueData(element.text || '');
        setUpArrow(false);
        setSearch('');
      }
    };
    const deleteTagSelected = (tag: DataValue) => {
      const _values: DataValue[] = [...valuesData].filter(e => e.value !== tag.value);
      onChangeValues && onChangeValues(_values);
      setValuesData(_values);
      setUpArrow(true);
    };
    const renderList = () => {
      let listDataToFilter = [...data || []];
      listDataToFilter = listDataToFilter.filter((element: DataValue) => element?.text?.toLowerCase().includes(search.toLowerCase()));
      if (prerequisite) return (
        <div style={{ textAlign: 'center' }}>
          {prerequisiteText}
        </div>
      );

      if (typeof max === 'number' && valuesData.length !== 0 && max === valuesData.length) return (
        <div>
          <Error position='center' title={maxText} />
        </div>
      );

      if (listDataToFilter.length === 0) return (
        <div style={{ textAlign: 'center' }}>
          {emptyText}
        </div>
      );

      if (multiple) {
        return listDataToFilter
          .map(e => (
            <div key={e.key} onClick={() => handleChange(e)} className={classes.results}>
              <p>{e.text}</p>
            </div>
          ));
      }

      return listDataToFilter.map((element) => (
        <div key={element.key} className={classes.results}>
          <p onClick={() => handleChange(element)} >{element.text}</p>
        </div>
      ));
    };
    const listTagsSelected = () => {
      if (valuesData && valuesData.length > 0) {
        return (
          <div
            style={{ borderBottom: upArrow ? `1px solid ${theme.bluePrimary}` : `1px solid ${theme.borderGreyPrimary}` }}
            className={classes.tagsContainer}
          >
            {valuesData.map((tag: DataValue) => (
              <div key={tag.key} className={classes.tag}>
                <p className={classes.tagTitle}>{tag.text}</p>
                <p className={classes.crossTag} onClick={() => deleteTagSelected(tag)}>
                  x
                </p>
              </div>
            ))}
          </div>
        )
      }
    };

    useEffect(() => {
      if (!multiple) {
        if (value === '') setValueData('');
        const index = data?.findIndex(e => e.value === value);
        if (Number.isInteger(index) && index !== -1) {
          const _value = Array.isArray(data) ? data[index || 0].text : '';
          setValueData(_value || '');
        }
      }
    }, [value]);

    useEffect(() => {
      if (multiple) {
        if (Array.isArray(values)) {
          let _values: DataValue[] = [];
          let selectedFilter: DataValue[] = [];
          values.forEach(v => {
            selectedFilter = data?.filter(e => e.value === v.value) || [];
            _values = _values.concat(selectedFilter);
          });
          setValuesData(_values);
        }
      }
    }, [values]);

    return (
      <div >
        {upArrow && (<div className={classes.backWall} onClick={() => setUpArrow(false)} />)}
        <div {...props} className={classes.root} style={{ border: upArrow ? `1px solid ${theme.bluePrimary}` : `1px solid ${theme.borderGreyPrimary}` }} >
          <TextField
            className={classes.field}
            id={id}
            type={type}
            value={valueData || ''}
            label={label}
            autoComplete="off"
            disabled
            name={name}
            InputProps={{
              endAdornment:
                (
                  <InputAdornment position="start">
                    <div style={{ paddingRight: '12px' }} onClick={() => setUpArrow(!upArrow)}>
                      {
                        !upArrow ?
                          <IconButton style={{ width: 36, height: 36 }}>
                            <KeyboardArrowDownIcon className={classes.arrow} style={{ border: `2px solid ${StoryTheme.borderGreyPrimary}` }} />
                          </IconButton>
                          :
                          <IconButton style={{ width: 36, height: 36 }}>
                            <KeyboardArrowUpIcon className={classes.arrow} style={{ border: `1px solid ${StoryTheme.bluePrimary}` }} />
                          </IconButton>
                      }
                    </div>
                  </InputAdornment>
                )
            }}
          />
          {multiple && <>{listTagsSelected()}</>}
          {upArrow && (
            <div className={classes.selector}>
              <div className={classes.inputSearch}>
                {!hideSearch && (
                  <TextField
                    className={classes.fieldSearch}
                    id={id}
                    type={type}
                    value={search || null}
                    onChange={handleInputFilter}
                    label={label}
                    fullWidth
                    autoComplete="off"
                    disabled={disabled}
                    name={name}
                    autoFocus
                    InputProps={{
                      endAdornment:
                        (
                          <InputAdornment position="start">
                            <div style={{ paddingRight: '12px' }}>
                              <SearchIcon className={classes.arrow} />
                            </div>
                          </InputAdornment>
                        )
                    }}
                  />
                )}
                <div className={classes.containerResults}>
                  {renderList()}
                </div>
              </div>
            </div>
          )}
        </div>
      </div>
    );
  };

const styles: any = makeStyles({
  backWall: {
    position: 'fixed',
    width: '100%',
    height: '100%',
    zIndex: 998,
    left: 0,
    top: 0,
    right: 0,
  },
  root: {
    display: 'flex',
    flexDirection: 'column',
    alignSelf: 'center',
    width: (props: any) => {
      return props.width || 'max-content'
    },
    borderRadius: 6,
    position: 'relative',
    zIndex: (props: any) => props.zIndex,
    '& > .MuiPaper-elevation3': {
      boxShadow: 'none',
      backgroundColor: 'white',
    },
  },
  field: {
    borderBottom: (props: any) => props.borderBottom,
    outline: 'none',
    borderRadius: 4,
    '& > .MuiInputBase-root': {
      marginTop: '1px',
      fontWeight: 300,
      fontSize: 16,
      borderRadius: '4px',
      '& > .MuiInputBase-input': {
        paddingLeft: '20px',
        paddingRight: '20px',
        paddingTop: '24px',
        paddingBottom: '15px',
        color: theme.blackPrimary,
        padding: '0px',
        borderRadius: '4px',
        backgroundColor: "transparent !important",
        zIndex: 100,
        '&:focus': {
          border: 'none',
        },
      },
    },
    '& > .MuiInput-root': {
      backgroundColor: theme.whitePrimary,
    },
    '& > .MuiInput-underline:before': {
      borderBottom: 'none',
      content: 'none',
    },
    '& > .MuiInput-underline:after': {
      backgroundColor: 'white',
      borderBottom: 'none',
    },
    '& > .MuiInput-underline:hover': {
      backgroundColor: theme.whitePrimary,
    },
    '& > .MuiInputLabel-animated': {
      color: 'grey',
      zIndex: 99,
      marginTop: 0,
      paddingTop: 8,
      paddingLeft: 15,
      fontSize: 16,
      transition:
        'color 200ms cubic-bezier(0.0, 0, 0.2, 1) 0ms,transform 200ms cubic-bezier(0.0, 0, 0.2, 1) 0ms',
    },
    '& > .MuiInputLabel-root:before': {
      color: theme.bluePrimary,
    },
  },
  fieldSearch: {
    border: `1px solid ${theme.bluePrimary}`,
    width: '100%',
    outline: 'none',
    borderRadius: 4,
    postion: 'relative',
    '& > .MuiInputBase-root': {
      marginTop: 1,
      borderRadius: 4,
      '& > .MuiInputBase-input': {
        paddingLeft: 20,
        paddingRight: 20,
        paddingTop: 24,
        paddingBottom: 15,
        color: '#000c24',
        padding: 0,
        borderRadius: 4,
        backgroundColor: 'transparent',
        zIndex: (props: any) => props.zIndex,
        '&:focus': {
          border: 'none',
        },
      },
    },
    '& > .MuiInput-root': {
      backgroundColor: theme.whitePrimary,
    },
    '& > .MuiInput-underline:before': {
      borderBottom: 'none',
      content: 'none',
    },
    '& > .MuiInput-underline:after': {
      backgroundColor: theme.whitePrimary,
      borderBottom: 'none',
    },
    '& > .MuiInput-underline:hover': {
      backgroundColor: theme.whitePrimary,
    },
    '& > .MuiInputLabel-animated': {
      color: 'grey',
      zIndex: 99,
      marginTop: 0,
      paddingTop: 8,
      paddingLeft: 15,
      fontSize: 16,
      transition:
        'color 200ms cubic-bezier(0.0, 0, 0.2, 1) 0ms,transform 200ms cubic-bezier(0.0, 0, 0.2, 1) 0ms',
    },
    '& > .MuiInputLabel-root:before': {
      color: 'blue',
    },
  },
  arrow: {
    color: theme.defaultSubtitle,
    width: '30px !important',
    height: '30px !important',
    borderRadius: '50%',
    cursor: 'pointer',
    border: `1px solid ${theme.borderGreyPrimary}`
  },
  selector: {
    border: (props: any) => `1px solid ${props.upArrow ? theme.bluePrimary : theme.borderGreyPrimary}`,
    minHeight: 190,
    backgroundColor: theme.whitePrimary,
    position: 'absolute',
    top: '100%',
    zIndex: 99999,
    minWidth: 'calc(100%)',
    overflowX: 'hidden',
    overflowY: 'auto',
  },
  inputSearch: {
    padding: 25,
  },
  containerResults: {
    maxHeight: 150,
    height: 'min-content',
    overflowX: 'auto',
    marginTop: 15,
    '&::-webkit-scrollbar': {
      width: "5px",
      backgroundColor: theme.grayPrimary,
    },
    '&::-webkit-scrollbar-thumb': {
      backgroundColor: theme.opacityGrayDefault,
      borderRadius: "20px",
    },
    "&::-webkit-scrollbar-track": {
      marginBottom: "1px",
    },
  },
  results: {
    fontSize: 15,
    '&>p': {
      padding: '6px 6px 6px 4px',
      margin: 0,
      '&:hover': {
        backgroundColor: theme.hoverBlue,
        cursor: 'pointer',
      },
    },
  },
  noResults: {
    textAlign: 'center',
    marginTop: 15,
  },
  errorMaxTags: {
    display: 'flex',
    justifyContent: 'center',
    '&>div': {
      width: 300,
      textAlign: 'center',
    },
  },
  containerTags: {
    border: `1px solid ${theme.borderGreyPrimary}`,
    marginTop: 10,
    padding: 15,
  },
  subContainerTags: {
    display: 'flex',
    flexFlow: 'wrap',
  },
  tag: {
    backgroundColor: theme.grayPrimary,
    borderRadius: 4,
    margin: 10,
    width: 'max-content',
    padding: 8,
    display: 'flex',
  },
  tagTitle: {
    fontSize: 14,
  },
  crossTag: {
    marginLeft: 20,
    '&:hover': {
      fontWeight: 700,
      cursor: 'pointer',
    },
  },
  tagsContainer: {
    display: 'flex',
    flexWrap: 'wrap',
    borderLeft: theme.borderGreyPrimary,
    borderRight: theme.borderGreyPrimary,
  },
  searchIcon: {
    border: `1px solid ${theme.borderGreyPrimary}`,
    width: 30,
    height: 30,
    padding: 5,
    borderRadius: '50%'
  }
});
