import React, { useState, useEffect, lazy, Suspense } from 'react';
import { makeStyles } from "@material-ui/core";
import { theme } from './Theme/theme';

import { PreviewPDF } from './PreviewPDF';
import { FilePreviewerThumbnail } from 'react-file-previewer';
const FileViewer = lazy(() => import('react-file-viewer-fix'))
import ReactAudioPlayer from 'react-audio-player';
import fileSaver from 'file-saver';
import { ThemeTypes } from './Theme/theme.types';

import { DetectFileType, FormatName, FileSize, DescriptionSize } from './helpers/fileTypes';

/* Icons */
import IconButton from '@material-ui/core/IconButton';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import {
  faFilePdf,
  faFileWord,
  faFileZipper,
  faFileExcel,
  faFileAudio,
  faFileVideo,
  faMagnifyingGlass,
  faDownload,
  faMagnifyingGlassPlus,
  faMagnifyingGlassMinus
} from '@fortawesome/free-solid-svg-icons';


type RecentFilesListType = {
  id: string;
  name: string;
  url: string;
  type: string;
  size?: string;
  created?: Date | string;
}

interface RenderFileTypes {
  [key: string]: any;
  file: RecentFilesListType;
  renderType: string;
  handlePreview: (v: RecentFilesListType) => void;
  selectedFile: RecentFilesListType;
  lng?: string;
}

export const RenderFile = ({
  file,
  renderType,
  handlePreview,
  selectedFile,
  lng,
  ...props }: RenderFileTypes) => {

  const styles = makeStyles({
    contaierSmallPreview: {
      position: 'relative',
      '&:hover': {
        '&>:nth-child(1)': {
          display: 'flex',
        }
      },
      '&>:nth-child(1)': {
        position: 'absolute',
        width: 50,
        height: 50,
        borderRadius: 4,
        border: `1px solid ${theme.grayPrimary}`,
        zIndex: 500,
        backgroundColor: theme.whiteOpacity,
        display: 'none',
        '&>svg': {
          color: theme.defaultSubtitle,
          padding: 5,
          margin: 5,
          width: 30,
          height: 30,
        },
        '&:hover': {
          border: `1px solid ${theme.bluePrimary}`,
          cursor: 'pointer'
        }
      }
    },
    previewFile: {
      '&>div': {
        width: '100%',
        height: '100%',
        '&>img': {
          width: (props: ThemeTypes) => props.image ? `${props.zoomValue}%` : '100%',
          height: (props: ThemeTypes) => props.image ? `${props.zoomValue}%` : '100%',
        },
      },
    },
    containerWithIcon: {
      display: 'flex',
      justifyContent: 'center',
      alignItems: 'center',
      padding: 4,
      '&>svg': {
        width: 50,
        height: 50,
        color: theme.opacityGrayDefault
      }
    },
    containerXLSX: {
      "&>div": {
        width: 'fit-content',
        "&>div": {
          "&>div": {
            "&>div": {
              "&>div": {
                "&>div": {
                  overflow: 'hidden !important',
                  maxHeight: 500,
                  "&>div": {
                    maxHeight: 500,
                    "&>div": {
                      display: 'flex',
                      flexFlow: 'column',
                      width: 'fit-content',
                      minWidth: '100vw',
                      overflow: 'hidden !important',
                      "&>div": {
                        overflow: 'hidden !important',
                        "&>div": {
                          '&::-webkit-scrollbar': {
                            width: "5px",
                            height: "7px",
                            backgroundColor: theme.grayPrimary,
                          },
                          '&::-webkit-scrollbar-thumb': {
                            backgroundColor: theme.opacityGrayDefault,
                            borderRadius: "20px",
                          },
                          "&::-webkit-scrollbar-track": {
                            marginBottom: "1px",
                          },
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    },
    menu: {
      borderTop: `2px solid ${theme.grayPrimary}`,
      backgroundColor: 'rgba(0,0,0,0.1)',
      display: 'flex',
      flexFlow: 'wrap',
      justifyContent: 'space-between'
    },
    infoModalFile: {
      marginLeft: 10,
      textAlign: 'left',
      fontSize: 14,
      color: theme.blackPrimary,
      display: 'flex',
      flexFlow: 'column',
      alignItems: 'center',
      marginTop: 4,
      '&>:nth-child(1)': {
        fontSize: 16,
        fontWeight: 700
      }
    },
    iconsZoom: {
      display: 'flex',
      flexFolow: 'wrap',
      justifyContent: 'flex-end',
      alignItems: 'center',
      '&>button': {
        width: 40,
        height: 40,
        margin: 10,
        display: 'flex',
        flexFolow: 'wrap',
        justifyContent: 'center',
        alignItems: 'center',
        '&>span': {
          color: theme.defaultSubtitle,
          padding: 5,
          margin: 5,
          width: 20,
          height: 20,
          '&:hover': {
            color: theme.bluePrimary,
            cursor: 'pointer'
          }
        }
      }
    },
    containerDOCX: {
      "&>div": {
        width: 'fit-content',
        overflow: 'hidden'
      }
    }
  });
  const [zoom, setZoom] = useState(100);
  const [isImage, setIsImage] = useState(false);
  const [isDocument, setIsDocument] = useState(false);
  const classes = styles({ zoomValue: zoom, image: isImage, docx: isDocument });

  const IsImage = () => {
    if (selectedFile.type !== '') {
      const type = DetectFileType(selectedFile.type)
      switch (type) {
        case 'jpeg':
        case 'jpg':
        case 'gif':
        case 'png':
        case 'svg':
          return true
        default:
          return false
      }
    } else {
      return false
    }
  }
  const IsDocument = () => {
    if (selectedFile.type !== '') {
      const type = DetectFileType(selectedFile.type)
      switch (type) {
        case 'docx':
        case 'xlsx':
        case 'pdf':
          return true
        default:
          return false
      }
    } else {
      return false
    }
  }

  useEffect(() => {
    if (selectedFile.type !== '') {
      setIsImage(IsImage())
      setIsDocument(false)
    } else {
      setIsImage(false)
      setIsDocument(IsDocument())
    }
  }, [selectedFile])

  const handleDownload = (url: string, name: string) => {
    fileSaver.saveAs(url, name)
  }

  const IsNotMediaFile = (fileType: string) => {
    const ext = DetectFileType(fileType)
    switch (ext) {
      case 'ogg':
      case 'wav':
      case 'mpeg':
      case 'mp4':
        return false
      default:
        return true
    }
  }

  const handleZoomById = (action: string) => {
    setTimeout(() => {
      const ele = (document.getElementById(DetectFileType(selectedFile.type)) as HTMLDivElement)
      const viewer = (document.getElementById('pg-viewer') as HTMLDivElement)

      if (viewer && ele && action === 'in') {
        if (zoom >= 100) {
          setZoom(zoom + 10)
          ele.style.transform = `scale(${(zoom + 10) / 100})`
          viewer.style.width = "100%"
          viewer.style.height = "100%"

        }
      }
      if (ele && viewer && action === 'out') {
        if (zoom >= 100) {
          setZoom(zoom - 10)
          ele.style.transform = `scale(${(zoom - 10) / 100})`
          viewer.style.width = "100%"
          viewer.style.height = "100%"
        }
        if (zoom === 100) {
          setZoom(100)
          ele.style.transform = `scale(${zoom / 100})`
          viewer.style.width = "100%"
          viewer.style.height = "100%"
        }
      }
    }, 300)
  }
  const handleZoom = (action: string) => {
    if (action === 'in') {
      if (zoom >= 100) setZoom(zoom + 10)
    }
    if (action === 'out') {
      if (zoom >= 100) setZoom(zoom - 10)
      if (zoom === 100) setZoom(100)
    }
  }

  const Menu = () => {
    return (
      <div className={classes.menu}>
        <div className={classes.infoModalFile}>
          <p>{FormatName(selectedFile.name)}</p>
          <p>{DescriptionSize(FileSize(selectedFile.size), lng || "")}</p>
        </div>
        <div className={classes.iconsZoom}>
          {IsNotMediaFile(selectedFile.type) && <IconButton onClick={() => handleDownload(selectedFile.url, selectedFile.name)}>
            <FontAwesomeIcon icon={faDownload} />
          </IconButton>}
          <IconButton onClick={() => isImage ? handleZoom('in') : handleZoomById('in')}>
            <FontAwesomeIcon icon={faMagnifyingGlassPlus} />
          </IconButton>
          <IconButton onClick={() => isImage ? handleZoom('out') : handleZoomById('out')}>
            <FontAwesomeIcon icon={faMagnifyingGlassMinus} />
          </IconButton>
        </div>
      </div>
    )
  }

  const FilesPreview = () => {
    const ext = DetectFileType(file.type)
    switch (ext) {
      case 'jpeg':
      case 'png':
      case 'bmp':
      case 'gif':
      case 'jpg':
      case 'svg':
        return (
          renderType === 'icon' ?
            (
              <div className={classes.contaierSmallPreview} {...props}>
                <div onClick={() => handlePreview(file)}>
                  <FontAwesomeIcon icon={faMagnifyingGlass} />
                </div>
                <FilePreviewerThumbnail
                  file={{ url: file.url }}
                />
              </div>
            )
            : (
              <React.Fragment>
                <div className={classes.previewFile} {...props}>
                  <FilePreviewerThumbnail
                    file={{ url: file.url }}
                  />
                </div>
                {Menu()}
              </React.Fragment>
            )
        )
      case 'pdf':
        return (
          renderType === 'icon' ?
            (
              <div className={classes.contaierSmallPreview} {...props}>
                <div onClick={() => handlePreview(file)}>
                  <FontAwesomeIcon icon={faMagnifyingGlass} />
                </div>
                <div className={classes.containerWithIcon}>
                  <FontAwesomeIcon icon={faFilePdf} />
                </div>
              </div>
            )
            : (
              <React.Fragment>
                <PreviewPDF
                  id={selectedFile.id}
                  name={FormatName(selectedFile.name)}
                  url={selectedFile.url}
                  size={DescriptionSize(FileSize(selectedFile.size), lng || "")}
                  created={selectedFile.created}
                  lng={lng}
                  type={selectedFile.type}
                />
              </React.Fragment>
            )
        )
      case 'xlsx':
        return (
          renderType === 'icon' ?
            (
              <div className={classes.contaierSmallPreview} {...props}>
                <div onClick={() => handlePreview(file)}>
                  <FontAwesomeIcon icon={faMagnifyingGlass} />
                </div>
                <div className={classes.containerWithIcon}>
                  <FontAwesomeIcon icon={faFileExcel} />
                </div>
              </div>
            )
            : (
              <React.Fragment>
                <div className={classes.containerXLSX} {...props}>
                  <div>
                    <Suspense fallback={<React.Fragment></React.Fragment>}>
                      <FileViewer
                        fileType={'xlsx'}
                        filePath={file.url}
                        errorComponent={false}
                        onError={false}
                      />
                    </Suspense>
                  </div>
                </div>
                {Menu()}
              </React.Fragment>
            )
        )
      case 'docx':
        return (
          renderType === 'icon' ?
            (
              <div className={classes.contaierSmallPreview} {...props}>
                <div onClick={() => handlePreview(file)}>
                  <FontAwesomeIcon icon={faMagnifyingGlass} />
                </div>
                <div className={classes.containerWithIcon}>
                  <FontAwesomeIcon icon={faFileWord} />
                </div>
              </div>

            )
            : (
              <React.Fragment>
                <div className={classes.containerDOCX}>
                  <Suspense fallback={<React.Fragment></React.Fragment>}>
                    <FileViewer
                      fileType={'docx'}
                      filePath={file.url}
                      errorComponent={false}
                      onError={false}
                    />
                  </Suspense>
                </div>
                {Menu()}
              </React.Fragment>
            )
        )
      case 'zip':
        return (
          renderType === 'icon' ?
            (
              <div className={classes.contaierSmallPreview} {...props}>
                <div onClick={() => handleDownload(file.url, file.name)}>
                  <FontAwesomeIcon icon={faDownload} />
                </div>
                <div className={classes.containerWithIcon}>
                  <FontAwesomeIcon icon={faFileZipper} />
                </div>
              </div>
            )
            : <React.Fragment />

        )
      case 'ogg':
      case 'wav':
      case 'mpeg':
        return (
          renderType === 'icon' ?
            (
              <div className={classes.contaierSmallPreview} {...props}>
                <div onClick={() => handlePreview(file)}>
                  <FontAwesomeIcon icon={faMagnifyingGlass} />
                </div>
                <div className={classes.containerWithIcon}>
                  <FontAwesomeIcon icon={faFileAudio} />
                </div>
              </div>
            )
            : (
              <div style={{ backgroundColor: theme.grayPrimary }} {...props}>
                <ReactAudioPlayer
                  src={file.url}                 
                  controls
                />
                {Menu()}
              </div>
            )

        )
      case 'mp4':
        return (
          renderType === 'icon' ?
            (
              <div className={classes.contaierSmallPreview} {...props}>
                <div onClick={() => handlePreview(file)}>
                  <FontAwesomeIcon icon={faMagnifyingGlass} />
                </div>
                <div className={classes.containerWithIcon}>
                  <FontAwesomeIcon icon={faFileVideo} />
                </div>
              </div>
            )
            : (
              <div {...props}>
                <video id="video-chat-messages-room" src={file.url} controls width="100%" height="100%">
                  <source src={file.url} type="mp4" />
                  Your browser does not support HTML5 video.
                </video>
              </div>
            )
        )
      default:
        <></>
    }
  }
  return <React.Fragment>{FilesPreview()}</React.Fragment>
}
